@@ -1,15 +1,15 @@
 import re
 import sys
 import logging
+import warnings
 from argparse import ArgumentParser, Action
 from typing import List
-
 from quid.passager.Passager import Passager
 from quid.passager.TextWithMatches import TextWithMatches
 from quid.core.Quid import Quid
 import json
 import time
-from os.path import join, isfile, splitext, basename
+from os.path import join, isfile, splitext, basename, exists
 from os import listdir
 from shutil import copyfile
 import multiprocessing
@@ -29,10 +29,7 @@ from quid.visualization.Visualizer import Visualizer
 from quid.helper.Loader import load_matches, load_citation_sources, load_citation_source_links
 
 
-logging.getLogger().setLevel(logging.INFO)
-
-
-class IntCheckAction(Action):
+class OptionValueCheckAction(Action):
 
     def __call__(self, parser, namespace, values, option_string=None):
 
@@ -54,6 +51,12 @@ class IntCheckAction(Action):
         elif option_string == '--max-num-processes':
             if int(values) <= 0:
                 parser.error("{0} must be greater 0".format(option_string))
+        elif option_string == 'min_levenshtein_distance':
+            if float(values) < 0 or float(values) > 1:
+                parser.error("{0} must be between 0 and 1".format(option_string))
+        elif option_string == 'split_length':
+            if int(values) <= 0:
+                parser.error("{0} must be greater 0".format(option_string))
 
         setattr(namespace, self.dest, values)
 
@@ -141,12 +144,11 @@ def __json_encoder(obj, strip):
 
 def __run_compare(source_file_path, target_path, export_text, output_type, csv_sep, min_match_length, look_ahead_limit,
                   look_back_limit, max_merge_distance, max_merge_ellipsis_distance, output_folder_path,
-                  num_of_processes, keep_ambiguous_matches):
+                  num_of_processes, keep_ambiguous_matches, min_levenshtein_similarity, split_long_texts, split_length):
     with open(source_file_path, 'r', encoding='utf-8') as source_file:
         source_file_content = source_file.read()
 
     if isfile(target_path) and target_path.endswith(".txt"):
-
         with open(target_path, 'r', encoding='utf-8') as target_file:
             target_file_content = target_file.read()
 
@@ -154,8 +156,12 @@ def __run_compare(source_file_path, target_path, export_text, output_type, csv_s
 
         __process_file(source_file_content, target_file_content, export_text, output_type, csv_sep, min_match_length,
                        look_ahead_limit, look_back_limit, max_merge_distance, max_merge_ellipsis_distance,
-                       output_folder_path, filename, keep_ambiguous_matches, None, None)
+                       output_folder_path, filename, keep_ambiguous_matches, None, None, min_levenshtein_similarity,
+                       split_long_texts, num_of_processes, split_length)
     else:
+        if split_long_texts:
+            warnings.warn('Split long texts is ignored when the source text is compared with multiple target texts.')
+
         start_time = time.perf_counter()
         quid = Quid(min_match_length, look_back_limit, look_ahead_limit, max_merge_distance,
                     max_merge_ellipsis_distance, export_text, keep_ambiguous_matches)
@@ -179,7 +185,8 @@ def __run_compare(source_file_path, target_path, export_text, output_type, csv_s
                                                        look_back_limit, max_merge_distance,
                                                        max_merge_ellipsis_distance, output_folder_path,
                                                        filename, keep_ambiguous_matches, min_length_match_positions,
-                                                       hashes))
+                                                       hashes, min_levenshtein_similarity, False,
+                                                       num_of_processes, split_length))
 
         pool.close()
         pool.join()
@@ -188,9 +195,10 @@ def __run_compare(source_file_path, target_path, export_text, output_type, csv_s
 def __process_file(source_file_content, target_file_content, export_text, output_type, csv_sep, min_match_length,
                    look_ahead_limit, look_back_limit, max_merge_distance, max_merge_ellipsis_distance,
                    output_folder_path, filename, keep_ambiguous_matches, cached_min_length_match_positions,
-                   cached_hashes):
+                   cached_hashes, min_levenshtein_similarity, split_long_texts, num_of_processes, split_length):
     quid = Quid(min_match_length, look_back_limit, look_ahead_limit, max_merge_distance, max_merge_ellipsis_distance,
-                export_text, keep_ambiguous_matches)
+                export_text, keep_ambiguous_matches, min_levenshtein_similarity, split_long_texts,
+                max_num_processes=num_of_processes, split_length=split_length, show_progress=True)
     matches = quid.compare(source_file_content, target_file_content, cached_min_length_match_positions, cached_hashes)
 
     if not export_text:
@@ -240,6 +248,7 @@ def __process_file(source_file_content, target_file_content, export_text, output
 
     if output_folder_path:
         filename = f'{filename}.{file_ending}'
+
         with open(join(output_folder_path, filename), 'w', encoding='utf-8') as output_file:
             output_file.write(result)
 
@@ -351,8 +360,12 @@ def main(argv=None):
                           "text. If known, the source text should be the one that is quoted by the target text. " \
                           "This allows the algorithm to handle things like ellipsis in quotations."
 
-    argument_parser = ArgumentParser(description="Quid is a tool to find quotations in texts and to visualize the"
-                                                 " matching segments.")
+    argument_parser = ArgumentParser(prog='quid', description="Quid is a tool to find quotations in texts and to"
+                                                              " visualize the matching segments.")
+
+    argument_parser.add_argument("--log-level", dest="log_level", choices=['DEBUG', 'INFO', 'WARNING', 'ERROR',
+                                                                           'CRITICAL'],
+                                 help="Set the logging level", default="WARNING")
 
     subparsers = argument_parser.add_subparsers(dest='command')
     subparsers.required = True
@@ -374,34 +387,48 @@ def main(argv=None):
     parser_compare.add_argument('--output-folder-path', dest="output_folder_path",
                                 help="The output folder path. If this option is set the output will be saved to a file"
                                      " created in the specified folder")
-    parser_compare.add_argument('--min-match-length', dest="min_match_length", action=IntCheckAction,
-                                default=5, type=int, help="The length of the shortest match (>= 1, default: 5)")
-
-    parser_compare.add_argument('--look-back-limit', dest="look_back_limit", action=IntCheckAction,
-                                default=10, type=int, help="The number of tokens to skip when looking backwards"
-                                                           " (>= 0, default: 10), (Very rarely needed)")
-    parser_compare.add_argument('--look-ahead-limit', dest="look_ahead_limit", action=IntCheckAction,
-                                default=3, type=int, help="The number of tokens to skip when looking ahead"
-                                                          " (>= 0, default: 3)")
-    parser_compare.add_argument('--max-merge-distance', dest="max_merge_distance", action=IntCheckAction,
+    parser_compare.add_argument('--min-match-length', dest="min_match_length", action=OptionValueCheckAction,
+                                default=5, type=int, help="The minimum number of tokens of a match (>= 1, default: 5)")
+    parser_compare.add_argument('--look-back-limit', dest="look_back_limit", action=OptionValueCheckAction,
+                                default=10, type=int, help="The maximum number of tokens to skip when extending a match"
+                                                           " backwards (>= 0, default: 10)")
+    parser_compare.add_argument('--look-ahead-limit', dest="look_ahead_limit", action=OptionValueCheckAction,
+                                default=3, type=int, help="The maximum number of tokens to skip when extending a match"
+                                                          " forwards (>= 0, default: 3)")
+    parser_compare.add_argument('--max-merge-distance', dest="max_merge_distance", action=OptionValueCheckAction,
                                 default=2, type=int, help="The maximum distance in tokens between two matches"
                                                           " considered for merging (>= 0, default: 2)")
     parser_compare.add_argument('--max-merge-ellipsis-distance', dest="max_merge_ellipsis_distance",
-                                action=IntCheckAction, default=10, type=int,
+                                action=OptionValueCheckAction, default=10, type=int,
                                 help="The maximum distance in tokens between two matches considered for merging where"
                                      " the target text contains an ellipsis between the matches (>= 0, default: 10)")
-    parser_compare.add_argument('--create-dated-subfolder', dest="created_dated_subfolder", default=False,
+    parser_compare.add_argument('--create-dated-subfolder', dest="create_dated_subfolder", default=False,
                                 action='store_true',
                                 help="Create a subfolder named with the current date to store the results")
-    parser_compare.add_argument('--no-create-dated-subfolder', dest="created_dated_subfolder",
+    parser_compare.add_argument('--no-create-dated-subfolder', dest="create_dated_subfolder",
                                 action='store_false',
                                 help="Don't create a subfolder named with the current date to store the results")
-    parser_compare.add_argument('--max-num-processes', dest="max_num_processes", action=IntCheckAction, default=1,
-                                type=int, help="Maximum number of processes to use for parallel processing")
+    parser_compare.add_argument('--max-num-processes', dest="max_num_processes", action=OptionValueCheckAction,
+                                default=1, type=int, help="Maximum number of processes to use for parallel processing")
     parser_compare.add_argument('--keep-ambiguous-matches', dest="keep_ambiguous_matches", default=False,
-                                action='store_true', help="Keep ambiguous matches")
+                                action='store_true', help="For a match with multiple matched segments in the source"
+                                                          " text, multiple matches will be returned.")
     parser_compare.add_argument('--no-keep-ambiguous-matches', dest='keep_ambiguous_matches', action='store_false',
-                                help="Don't ambiguous matches")
+                                help="For a match with multiple matched segments in the source text, only the first"
+                                     " match will be returned.")
+    parser_compare.add_argument('--min-levenshtein-similarity', dest="min_levenshtein_similarity",
+                                action=OptionValueCheckAction, default=0.85, type=float,
+                                help="The threshold for the minimal levenshtein similarity between tokens (and the"
+                                     " initial n-grams) to be accepted as a match (between 0 and 1, default: 0.85)")
+    parser_compare.add_argument('--split-long-texts', dest="split_long_texts", default=False,
+                                action='store_true', help="Split texts longer than split-length words for faster"
+                                                          "processing")
+    parser_compare.add_argument('--no-split-long-texts', dest='split_long_texts', action='store_false',
+                                help="Do not split texts longer than 30000 tokens.")
+    parser_compare.add_argument('--split-length', dest="split_length", action=OptionValueCheckAction,
+                                default=30000, type=int, help="If split-long-texts is set to True, texts longer (in"
+                                                              " number of words) than this threshold will be split for"
+                                                              " faster processing.")
 
     passage_description = 'Quid passage allows the user to extract key passages from the found matches.'
 
@@ -439,6 +466,9 @@ def main(argv=None):
 
     args = argument_parser.parse_args(argv)
 
+    log_level = args.log_level
+    logging.getLogger().setLevel(logging.getLevelName(log_level))
+
     if args.command == 'compare':
         source_path = args.source_file_path[0]
         target_path = args.target_path[0]
@@ -451,21 +481,29 @@ def main(argv=None):
         look_back_limit = args.look_back_limit
         max_merge_distance = args.max_merge_distance
         max_merge_ellipsis_distance = args.max_merge_ellipsis_distance
-        created_dated_subfolder = args.created_dated_subfolder
+        create_dated_subfolder = args.create_dated_subfolder
         max_num_processes = args.max_num_processes
         keep_ambiguous_matches = args.keep_ambiguous_matches
+        min_levenshtein_similarity = args.min_levenshtein_similarity
+        split_long_texts = args.split_long_texts
+        split_length = args.split_length
 
-        if output_folder_path and created_dated_subfolder:
-            now = datetime.now()
-            date_time_string = now.strftime("%Y_%m_%d_%H_%M_%S")
-            output_folder_path = join(args.output_folder_path, date_time_string)
-            Path(output_folder_path).mkdir(parents=True, exist_ok=True)
+        if output_folder_path:
+            if not exists(output_folder_path):
+                raise Exception(f'{output_folder_path} does not exist!')
+
+            if create_dated_subfolder:
+                now = datetime.now()
+                date_time_string = now.strftime("%Y_%m_%d_%H_%M_%S")
+                output_folder_path = join(args.output_folder_path, date_time_string)
+                Path(output_folder_path).mkdir(parents=True, exist_ok=True)
 
         start_time = time.perf_counter()
 
         __run_compare(source_path, target_path, export_text, output_type, csv_sep, min_match_length, look_ahead_limit,
                       look_back_limit, max_merge_distance, max_merge_ellipsis_distance, output_folder_path,
-                      max_num_processes, keep_ambiguous_matches)
+                      max_num_processes, keep_ambiguous_matches, min_levenshtein_similarity, split_long_texts,
+                      split_length)
 
         end_time = time.perf_counter()
         logging.info(f'\n--- Runtime: {end_time - start_time: .2f} seconds ---')
