@@ -1,11 +1,14 @@
+from dataclasses import dataclass
+
+
+@dataclass
 class SourceSegment:
-    def __init__(self, my_id, start, end, frequency, token_length, text):
-        self.my_id = my_id
-        self.start = start
-        self.end = end
-        self.frequency = frequency
-        self.token_length = token_length
-        self.text = text
+    my_id: int
+    start: int
+    end: int
+    frequency: int
+    token_length: int
+    text: str
 
     @classmethod
     def from_frequency(cls, my_id, start, end, frequency):
