@@ -1,8 +1,14 @@
+from dataclasses import dataclass
+from typing import List
+
+from quid.passager.TargetLocation import TargetLocation
+
+
+@dataclass
 class TargetText:
-    def __init__(self, my_id, filename, target_locations):
-        self.my_id = my_id
-        self.filename = filename
-        self.target_locations = target_locations
+    my_id: int
+    filename: str
+    target_locations: List[TargetLocation]
 
     def __hash__(self) -> int:  # pragma: no cover
         return hash((self.my_id, self.filename, hash(self.target_locations)))
